package hu.bme.mit.bookmanager.ui;

import hu.bme.mit.bookmanager.Book;
import hu.bme.mit.bookmanager.BookManager;
import hu.bme.mit.bookmanager.IBookManager;

import java.util.List;

import org.eclipse.core.databinding.Binding;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.conversion.IConverter;
import org.eclipse.core.databinding.conversion.NumberToStringConverter;
import org.eclipse.core.databinding.conversion.StringToNumberConverter;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.validation.IValidator;
import org.eclipse.core.databinding.validation.ValidationStatus;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.databinding.viewers.ViewersObservables;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.ColumnLayout;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.part.ViewPart;

public class BookDetailView extends ViewPart {
	private DataBindingContext m_bindingContext;
	public BookDetailView() {
	}

	private final class PagenumValidator implements IValidator {

		ControlDecoration decoration;

		public PagenumValidator(ControlDecoration controlDecoration) {
			this.decoration = controlDecoration;
		}

		@Override
		public IStatus validate(Object value) {
			int num = 0;
			boolean valid = true;
			try {
				num = Integer.parseInt((String) value);
			} catch (NumberFormatException e) {
				valid = false;
			}
			valid = valid && (num > 0);
			if (valid) {
				decoration.hide();
				return ValidationStatus.ok();
			} else {
				decoration.show();
				decoration
						.setDescriptionText("The page number has to be a positive number.");
				return ValidationStatus
						.error("The page number has to be a positive number.");
			}
		}

	}

	private final class StatusToColorConverter implements IConverter {

		@Override
		public Object convert(Object fromObject) {
			IStatus status = (IStatus) fromObject;
			if (status.getSeverity() == IStatus.ERROR)
				return Display.getCurrent().getSystemColor(SWT.COLOR_RED);
			return Display.getCurrent().getSystemColor(
					SWT.COLOR_WHITE);
		}

		@Override
		public Object getFromType() {
			return IStatus.class;
		}

		@Override
		public Object getToType() {
			return Color.class;
		}

	}

	FormToolkit toolkit;
	Form form;

	private Text author;
	private Text title;
	private Text pagenum;
	private Button read;

	private IBookManager manager;
	private IObservableValue selection;
	private TableViewer viewer;
	private ControlDecoration controlDecoration;
	private FieldDecoration fieldDecoration;
	private Composite detailsComposite;

	@Override
	public void createPartControl(Composite parent) {
		toolkit = new FormToolkit(parent.getDisplay());
		form = toolkit.createForm(parent);
		toolkit.decorateFormHeading(form);
		form.setText("Book Demo");
		Composite body = form.getBody();
		body.setLayout(new ColumnLayout());

		// Booklist section
		Section booksSection = toolkit.createSection(body, Section.DESCRIPTION
				| Section.TITLE_BAR | Section.EXPANDED);
		booksSection.setText("Books");
		Table table = toolkit.createTable(booksSection, SWT.FULL_SELECTION);
		booksSection.setClient(table);
		viewer = new TableViewer(table);
		viewer.setContentProvider(new IStructuredContentProvider() {
			
			IBookManager manager;
			
			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				if (newInput instanceof BookManager) {
					manager = (IBookManager) newInput;
				} else {
					//TODO error handling
					manager = null;
				}
				
			}
			
			@Override
			public void dispose() {}
			
			@Override
			public Object[] getElements(Object inputElement) {
				if (manager == null) {
					return new Object[0];
				} else {
					List<Book> books = manager.getAllBooks();
					return books.toArray(new Book[books.size()]);
				}
			}
		});
		TableViewerColumn authorColumn = new TableViewerColumn(viewer, SWT.LEFT);
		authorColumn.getColumn().setText("Author");
		authorColumn.setLabelProvider(new CellLabelProvider() {
			
			@Override
			public void update(ViewerCell cell) {
				Book book = (Book) cell.getElement();
				cell.setText(book.getAuthor());
				
			}
		});
		TableViewerColumn titleColumn = new TableViewerColumn(viewer, SWT.LEFT);
		titleColumn.setLabelProvider(new CellLabelProvider() {
			
			@Override
			public void update(ViewerCell cell) {
				Book book = (Book) cell.getElement();
				cell.setText(book.getTitle());
			}
		});
		titleColumn.getColumn().setText("Title");
		TableLayout tableLayout = new TableLayout();
		tableLayout.addColumnData(new ColumnWeightData(100, true));
		tableLayout.addColumnData(new ColumnWeightData(100, true));
		table.setLayout(tableLayout);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// Book details section
		Section detailsSection = toolkit.createSection(body,
				Section.DESCRIPTION | Section.TITLE_BAR | Section.EXPANDED);
		detailsSection.setText("Book Details");
		detailsSection.setDescription("Book Details");
		detailsComposite = toolkit.createComposite(detailsSection);
		detailsSection.setClient(detailsComposite);
		detailsComposite.setLayout(new GridLayout(2, true));
		toolkit.createLabel(detailsComposite, "Author");
		author = toolkit.createText(detailsComposite, "Author name");
		toolkit.createLabel(detailsComposite, "Title");
		title = toolkit.createText(detailsComposite, "Book title");
		toolkit.createLabel(detailsComposite, "PageNum");
		pagenum = toolkit.createText(detailsComposite, "Number of pages");
		read = toolkit.createButton(detailsComposite, "Read", SWT.CHECK);

		// Layouting
		{
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 150;
			author.setLayoutData(gd);
		}
		{
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 150;

			title.setLayoutData(gd);
		}
		{
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 150;
		}
		{
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.widthHint = 150;
			pagenum.setLayoutData(gd);
		}

		// Loading content
		manager = new BookManager();

		controlDecoration = new ControlDecoration(pagenum,
				SWT.LEFT | SWT.TOP);
		controlDecoration.setDescriptionText("hoverText");
		fieldDecoration = FieldDecorationRegistry.getDefault()
				.getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
		controlDecoration.setImage(fieldDecoration.getImage());

		
		m_bindingContext = initDataBindings();
		
		viewer.setSorter(new ViewerSorter(){

			@Override
			public int compare(Viewer viewer, Object e1, Object e2) {
				if (e1 instanceof Book && e2 instanceof Book) {
					Book o1 = (Book) e1;
					Book o2 = (Book) e2;
					int authorCompare = o1.getAuthor().compareTo(o2.getAuthor());
					if (authorCompare == 0) {
						return o1.getTitle().compareTo(o2.getTitle());
					} else {
						return authorCompare;
					}
				}
				return super.compare(viewer, e1, e2);
			}
			
		});
		viewer.addFilter(new ViewerFilter() {
			
			@Override
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				if (element instanceof Book) {
					return ((Book) element).isRead();
				} else {
					return true;
				}
			}
		});
		
		viewer.setInput(manager);
		if (!manager.getAllBooks().isEmpty()) {
			viewer.setSelection(new StructuredSelection(manager.getAllBooks().get(0)));
		}
	}

	@Override
	public void setFocus() {
		form.setFocus();

	}

	@Override
	public void dispose() {
		toolkit.dispose();
	}

	protected DataBindingContext initDataBindings() {
		DataBindingContext bindingContext = new DataBindingContext();
		//
		selection = ViewersObservables
				.observeSingleSelection(viewer);
		IObservableValue authorObservable = BeansObservables
				.observeDetailValue(selection, "author", String.class);
		IObservableValue titleObservable = BeansObservables.observeDetailValue(
				selection, "title", String.class);
		IObservableValue pageNumObservable = BeansObservables
				.observeDetailValue(selection, "pageNum", Integer.class);
		IObservableValue readObservable = BeansObservables.observeDetailValue(
				selection, "read", Boolean.class);
		DataBindingContext dbc = new DataBindingContext();
		UpdateValueStrategy updateStrategy = new UpdateValueStrategy(false,
				UpdateValueStrategy.POLICY_UPDATE);
		dbc.bindValue(SWTObservables.observeText(title, SWT.Modify),
				titleObservable, updateStrategy, updateStrategy);
		dbc.bindValue(SWTObservables.observeText(author, SWT.Modify),
				authorObservable);
		Binding pagenumBind = dbc.bindValue(SWTObservables.observeText(pagenum,
				SWT.Modify), pageNumObservable, new UpdateValueStrategy()
				.setConverter(StringToNumberConverter.toInteger(true)).setAfterGetValidator(
						new PagenumValidator(controlDecoration)),
				new UpdateValueStrategy()
						.setConverter(NumberToStringConverter.fromInteger(true)));
		dbc.bindValue(SWTObservables.observeSelection(read), readObservable);

		dbc.bindValue(SWTObservables.observeBackground(pagenum), pagenumBind
				.getValidationStatus(), null, new UpdateValueStrategy()
				.setConverter(new StatusToColorConverter()));
		new Label(detailsComposite, SWT.NONE);

		//
		return bindingContext;
	}
	
}
